#include "HT1632.h"

#define DATA 2
#define WR   3
#define CS   4
#define CS2  5

// use this line for single matrix
// HT1632LEDMatrix matrix = HT1632LEDMatrix(DATA, WR, CS);
// use this line for two matrices!
HT1632LEDMatrix matrix = HT1632LEDMatrix(DATA, WR, CS, CS2);

static prog_uint8_t fancyZero[] PROGMEM = {
  0x07, 0xE0, 0x1F, 0xF8, 0x3F, 0xFC, 0x3C, 0x3C,
  0x78, 0x1E, 0x78, 0x1E, 0x78, 0x1E, 0xF0, 0x0F,
  0xF0, 0x0F, 0xF0, 0x0F, 0xF0, 0x0F, 0xF0, 0x0F,
  0xF0, 0x0F, 0x78, 0x1E, 0x78, 0x1E, 0x78, 0x1E,
  0x3C, 0x3C, 0x3F, 0xFC, 0x1F, 0xF8, 0x07, 0xE0
};

static prog_uint8_t fancyOne[] PROGMEM = {
  0x01, 0xE0, 0x03, 0xE0, 0x07, 0xE0, 0x1F, 0xE0,
  0x1D, 0xE0, 0x01, 0xE0, 0x01, 0xE0, 0x01, 0xE0,
  0x01, 0xE0, 0x01, 0xE0, 0x01, 0xE0, 0x01, 0xE0,
  0x01, 0xE0, 0x01, 0xE0, 0x01, 0xE0, 0x01, 0xE0,
  0x01, 0xE0, 0x07, 0xF8, 0x07, 0xF8, 0x07, 0xF8
};

static prog_uint8_t fancyTwo[] PROGMEM = {
  0x0F, 0xF0, 0x3F, 0xF8, 0x7F, 0xFC, 0xFC, 0x3E,
  0xF8, 0x1F, 0xF0, 0x0F, 0xF0, 0x0F, 0x78, 0x0F,
  0x30, 0x1F, 0x00, 0x7E, 0x01, 0xFC, 0x07, 0xF0,
  0x1F, 0xC0, 0x3F, 0x00, 0x7C, 0x03, 0xF0, 0x0F,
  0xF7, 0xCF, 0xFF, 0xFF, 0xFF, 0xFE, 0xF0, 0xFE
};

static prog_uint8_t fancyThree[] PROGMEM = {
  0x3F, 0xF8, 0x7F, 0xFC, 0xFF, 0xFE, 0xF0, 0x1F,
  0xE0, 0x0F, 0x00, 0x0F, 0x00, 0x1F, 0x00, 0x3E,
  0x01, 0xFC, 0x01, 0xF8, 0x00, 0x3C, 0x00, 0x1E,
  0x00, 0x0F, 0x38, 0x0F, 0x70, 0x0F, 0xF0, 0x1F,
  0xF8, 0x3E, 0x7F, 0xFC, 0x3F, 0xF8, 0x0F, 0xF0
};

static prog_uint8_t fancyFour[] PROGMEM = {
  0x03, 0xC0, 0x03, 0xC0, 0x07, 0x80, 0x07, 0x80,
  0x0F, 0x00, 0x0F, 0x00, 0x1E, 0x00, 0x3C, 0x3C,
  0x7C, 0x3C, 0xF8, 0x3C, 0xF0, 0x3C, 0xFF, 0xFF,
  0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x3C, 0x00, 0x3C,
  0x00, 0x3C, 0x00, 0x3C, 0x00, 0x7E, 0x00, 0x7E
};

static prog_uint8_t fancyFive[] PROGMEM = {
  0x78, 0x00, 0x7F, 0xFF, 0x7F, 0xFF, 0x7F, 0xFF,
  0x78, 0x00, 0xF8, 0x00, 0xF1, 0xE0, 0xF7, 0xF8,
  0xFF, 0xFC, 0xFE, 0x3E, 0xF0, 0x1E, 0x00, 0x1F,
  0x00, 0x0F, 0x00, 0x0F, 0x38, 0x0F, 0x70, 0x1F,
  0xF0, 0x7E, 0xFF, 0xFC, 0x7F, 0xF8, 0x1F, 0xE0
};

static prog_uint8_t fancySix[] PROGMEM = {
  0x03, 0xF0, 0x0F, 0xE0, 0x1F, 0xC0, 0x3E, 0x00,
  0x3C, 0x00, 0x78, 0x00, 0x78, 0x00, 0xF1, 0xE0,
  0xF7, 0xF8, 0xFF, 0xFC, 0xFF, 0x3E, 0xFC, 0x1F,
  0xF8, 0x0F, 0xF0, 0x0F, 0xF0, 0x0F, 0xF8, 0x1F,
  0x7C, 0x3E, 0x3F, 0xFC, 0x1F, 0xF8, 0x0F, 0xF0 
};

static prog_uint8_t fancySeven[] PROGMEM = {
  0x70, 0x3F, 0x7F, 0xFF, 0xFF, 0xFF, 0xFF, 0xEF,
  0xF0, 0x0F, 0xF0, 0x1E, 0x00, 0x1E, 0x00, 0x3C,
  0x00, 0x3C, 0x00, 0x78, 0x00, 0xF0, 0x00, 0xF0,
  0x01, 0xE0, 0x01, 0xE0, 0x01, 0xE0, 0x03, 0xC0,
  0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0
};

static prog_uint8_t fancyEight[] PROGMEM = {
  0x1F, 0xF8, 0x3F, 0xFC, 0x7F, 0xFE, 0x78, 0x1E,
  0xF0, 0x0F, 0xF0, 0x0F, 0xF8, 0x1F, 0x7E, 0x7E,
  0x3F, 0xFC, 0x0F, 0xF0, 0x3F, 0xFC, 0x7E, 0x7E,
  0xF8, 0x1F, 0xF0, 0x0F, 0xF0, 0x0F, 0xF8, 0x1F,
  0x7C, 0x3E, 0x3F, 0xFC, 0x1F, 0xF8, 0x0F, 0xF0
};

static prog_uint8_t fancyTest[] PROGMEM = {
  0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
  0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08,
  0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10
};

void setup() {
  Serial.begin(9600);
  matrix.begin(HT1632_COMMON_16NMOS); 
}

void loop() {
  const long one_tick = 1000000L;
  long next_tick = 0L;
  int x=0;
  int displayWidth = matrix.width() - 1;
  int topX;
 
  matrix.clearScreen();
  matrix.setBrightness(1);
 
  while (true){

    switch(x/600) {
      case 0: matrix.drawBitmapRight(28, 0, fancyZero,  16, 20, 0x1); break;
      case 1: matrix.drawBitmapRight(28, 0, fancyOne,   16, 20, 0x1); break;
      case 2: matrix.drawBitmapRight(28, 0, fancyTwo,   16, 20, 0x1); break;
      case 3: matrix.drawBitmapRight(28, 0, fancyThree, 16, 20, 0x1); break;
      case 4: matrix.drawBitmapRight(28, 0, fancyFour,  16, 20, 0x1); break;
      case 5: matrix.drawBitmapRight(28, 0, fancyFive,  16, 20, 0x1); break;
      // we don't expect to go higher than 5, but just in case:
      case 6: matrix.drawBitmapRight(28, 0, fancySix,   16, 20, 0x1); break;
      case 7: matrix.drawBitmapRight(28, 0, fancySeven, 16, 20, 0x1); break;
      case 8: matrix.drawBitmapRight(28, 0, fancyEight, 16, 20, 0x1); break;
      // for 9, we use 6 upside down:
      case 9: matrix.drawBitmapLeft(28, 0,  fancySix,   16, 20, 0x1); break;
      // in case of a bad digit:
      default: matrix.clearScreen();
    }
    
    switch((x/60)%10) {
      case 0: matrix.drawBitmapLeft(0, 0, fancyZero,  16, 20, 0x1); break;
      case 1: matrix.drawBitmapLeft(0, 0, fancyOne,   16, 20, 0x1); break;
      case 2: matrix.drawBitmapLeft(0, 0, fancyTwo,   16, 20, 0x1); break;
      case 3: matrix.drawBitmapLeft(0, 0, fancyThree, 16, 20, 0x1); break;
      case 4: matrix.drawBitmapLeft(0, 0, fancyFour,  16, 20, 0x1); break;
      case 5: matrix.drawBitmapLeft(0, 0, fancyFive,  16, 20, 0x1); break;
      case 6: matrix.drawBitmapLeft(0, 0, fancySix,   16, 20, 0x1); break;
      case 7: matrix.drawBitmapLeft(0, 0, fancySeven, 16, 20, 0x1); break;
      case 8: matrix.drawBitmapLeft(0, 0, fancyEight, 16, 20, 0x1); break;
      // for 9, we use 6 upside down:
      case 9: matrix.drawBitmapRight(0, 0, fancySix,  16, 20, 0x1); break;
      // in case of a bad digit:
      default: matrix.clearScreen();
    }
    
    for (int i=0; i<9; i++) matrix.drawPixel(23, i, ((x%10)>i) ? 1 : 0);
    for (int i=0; i<5; i++) matrix.drawPixel(23, i+10, (((x/10)%6)>i) ? 1 : 0);
    
    matrix.writeScreen();

    next_tick += one_tick;
    //matrix.clearScreen();
    while ((long)(micros() - next_tick) < 0){
      Serial.println(next_tick);
      delay(5);
    }
    
    x = (x+1) % 3600;
  }
  // whew!
}
